import importlib

from zenplate.plugins.plugin_manager import PluginManager
from zenplate.plugins import (
    Plugin,
    plugin_wrapper,
    DataPlugin,
    JinjaTestPlugin,
    JinjaFilterPlugin,
)

from config_fixtures import fixtures


@plugin_wrapper("decorated_function", Plugin)
def decorated_function(unique_signature: str):
    return f"{unique_signature}"


@plugin_wrapper("data_plugin", DataPlugin)
def data_plugin(*args, **kwargs):
    return {"test": "test"}


@plugin_wrapper("jinja_test_plugin", JinjaTestPlugin)
def jinja_test_plugin(string: str):
    return string is not None and len(string) > 1


@plugin_wrapper("jinja_filter_plugin", JinjaFilterPlugin)
def jinja_filter_plugin(a_list: list):
    return ",".join(a_list)


def test_plugin_manager_find_plugins_by_module_name():
    plugin_manager = PluginManager(DataPlugin)
    plugins = plugin_manager.find_plugins_by_module_name(
        "zenplate_plugin_test.data_plugins"
    )
    assert "data" in plugins.keys()


def test_plugin_manager_find_matching_plugins():
    plugin_manager = PluginManager(DataPlugin)
    plugins = plugin_manager.find_matching_plugins()
    assert "data" in plugins.keys()


def test_plugin_manager_find_plugins_from_path():
    path_to_module = (
        fixtures / "plugin_module_for_matched_import" / "zenplate_plugin_test"
    )
    plugin_manager = PluginManager(DataPlugin)
    plugins = plugin_manager.find_plugins_from_path(str(path_to_module))
    assert "data" in plugins.keys()


def test_plugin_manager_find_plugins_from_module():
    plugin_manager = PluginManager(DataPlugin)
    module = importlib.import_module("zenplate_plugin_test.data_plugins")
    plugins = plugin_manager.find_plugins_from_module(module)
    assert "data" in plugins.keys()


def test_plugin_manager_specificity():
    plugin_manager = PluginManager(JinjaTestPlugin)
    module = importlib.import_module("zenplate_plugin_test.jinja_plugins")
    plugins = plugin_manager.find_plugins_from_module(module)
    assert "jinja_test" in plugins.keys()
    assert "jinja_filter" not in plugins.keys()


def test_multiple_plugin_managers():
    plugin_config = {
        "named_modules": [
            "zenplate_plugin_test.data_plugins",
            "zenplate_plugin_test.jinja_plugins",
        ],
    }
    data_plugin_manager = PluginManager(DataPlugin)
    data_plugin_manager.load_plugins(plugin_config)
    data_plugin_class = data_plugin_manager.plugins.get("data")
    data_plugin_result = data_plugin_class()

    assert data_plugin_result == "test"

    jinja_test_manager = PluginManager(JinjaTestPlugin)
    jinja_test_manager.load_plugins(plugin_config)
    jinja_test_plugin_class = jinja_test_manager.plugins.get("jinja_test")
    jinja_test_plugin_result = jinja_test_plugin_class("test")
    assert jinja_test_plugin_result

    jinja_filter_manager = PluginManager(JinjaFilterPlugin)
    jinja_filter_manager.load_plugins(plugin_config)
    jinja_filter_plugin_class = jinja_filter_manager.plugins.get("jinja_filter")
    jinja_filter_plugin_result = jinja_filter_plugin_class(["abc", "ab", "a"])

    assert jinja_filter_plugin_result == "['abc', 'ab']"


def test_plugin_manager_construct_plugins_dict():
    plugin_manager = PluginManager(DataPlugin)
    plugin_manager.construct_plugins_dict({"test": data_plugin})

    assert "data_plugin" in plugin_manager.plugins.keys()


if __name__ == "__main__":
    test_plugin_manager_find_plugins_by_module_name()
    test_plugin_manager_find_matching_plugins()
    test_plugin_manager_find_plugins_from_path()
    test_plugin_manager_find_plugins_from_module()
    test_plugin_manager_specificity()
    test_multiple_plugin_managers()
    test_plugin_manager_construct_plugins_dict()



from pathlib import Path
from zenplate.plugins import plugin_wrapper, DataPlugin


@plugin_wrapper("directory_listing", DataPlugin)
def directory_listing(path: str = "."):
    return (
        [i.name for i in Path(path).iterdir() if i.is_file()]
        if Path(path).is_dir()
        else []
    )



from typing import Optional, List
from typing_extensions import Annotated
from pathlib import Path
from enum import Enum

import typer
import click

from zenplate.config import Config
from zenplate.__main__ import main


cli = typer.Typer()


class LogLevels(str, Enum):
    debug = "DEBUG"
    info = "INFO"
    warning = "WARNING"
    error = "ERROR"
    critical = "CRITICAL"


@cli.command("", no_args_is_help=True)
def run(
    template: Annotated[
        Optional[Path],
        typer.Argument(
            help="The path to the jinja template / directory that zenplate will render",
            dir_okay=True,
            file_okay=True,
        ),
    ] = None,
    output: Annotated[
        Optional[Path],
        typer.Argument(
            help="The path to where you'll find the output of zenplate",
            dir_okay=True,
            file_okay=True,
        ),
    ] = None,
    config_file: Annotated[
        Optional[Path],
        typer.Option(
            help="The location of the yaml configuration file",
            show_default=True,
            dir_okay=False,
            envvar="ZENPLATE_CONFIG_FILE",
        ),
    ] = None,
    variables: Annotated[
        Optional[List[str]],
        typer.Option(
            help="A 'varname=value' pair representing a variable. May be used multiple times."
        ),
    ] = None,
    var_file: Annotated[
        Optional[List[Path]],
        typer.Option(
            help="The path to a yaml file containing key: value pairs to be used as variables. "
            "may be used multiple times.",
            dir_okay=False,
            envvar="ZENPLATE_VAR_FILE",
        ),
    ] = None,
    log_path: Annotated[
        Optional[Path],
        typer.Option(
            help="The location of the log file",
            dir_okay=False,
            envvar="ZENPLATE_LOG_PATH",
        ),
    ] = None,
    log_level: Annotated[
        Optional[LogLevels],
        typer.Option(
            help="The logging verbosity level",
            show_default=True,
            envvar="ZENPLATE_LOG_LEVEL",
        ),
    ] = LogLevels.error,
    export_config: Annotated[
        bool,
        typer.Option(
            help="When provided, the current set of configuration parameters will be exported "
            "to '--config-file' or './zenplate_config_export.yml' if not provided",
            is_flag=True,
            flag_value=True,
        ),
    ] = False,
    force: Annotated[
        bool,
        typer.Option(
            help="When provided, output will overwrite any file in that path",
            is_flag=True,
        ),
    ] = False,
    stdout: Annotated[
        bool,
        typer.Option(
            help="Write rendered template to stdout",
            is_flag=True,
        ),
    ] = False,
):
    if (not template and not output) and not export_config:
        raise click.exceptions.BadArgumentUsage(
            "You must provide both 'template' and 'output' arguments "
            "unless the '--help' or '--export-config' flags are provided."
        )
    if (template and not output) or (output and not template):
        raise click.exceptions.BadArgumentUsage(
            "You must provide both 'template' and 'output' arguments."
        )
    if (
        (template and output.exists())
        and (template and template.is_dir() and output and not output.is_dir())
        or (template and template.is_file() and output and not output.is_file())
    ):
        raise click.exceptions.BadArgumentUsage(
            "Template and output paths must both be either directories or files."
        )

    config = Config()
    if config_file:
        config.configure_from_path(config_file)
    if log_path:
        config.log_path = log_path
    if variables:
        config.variables = variables
    if var_file:
        config.var_files = [Path(i) for i in var_file]
    if template and template.is_dir():
        config.tree_directory = template
    elif template and template.is_file():
        config.template_path = template
    elif not template:
        pass
    else:
        raise click.exceptions.BadArgumentUsage(
            f"Template path '{template}' is not a file or directory."
        )
    if output:
        config.output_path = Path(output)
    if export_config:
        config.export_config()
        exit(0)

    config.force_overwrite = force
    config.stdout = stdout
    config.log_level = log_level

    main(config)


if __name__ == "__main__":
    cli()



from pathlib import Path
from typing import Type

import importlib
import importlib.util
import pkgutil
import inspect


from zenplate.plugins.base import Plugin


class PluginManager:
    plugin_config: dict
    plugin_modules: list
    base_class: Type[Plugin] = Plugin
    plugin_config: dict = {}

    def __init__(
        self,
        base_class: Type[Plugin] = Plugin,
        plugin_search_string: str = "zenplate_plugin",
    ):
        self._plugins = {}
        self.plugin_search_string = plugin_search_string
        self.base_class = base_class

    def load_plugins(self, plugin_config: dict):
        self.plugin_config = plugin_config
        path_modules = self.plugin_config.get("path_modules", [])

        for path_module in path_modules:
            if not Path(path_module).exists():
                raise FileNotFoundError(
                    f"Error loading plugin path module ({path_module}). Path does not exist."
                )
        found_plugins = {}
        for module in path_modules:
            found_plugins.update(self.find_plugins_from_path(module))

        named_modules = self.plugin_config.get("named_modules", [])
        for module in named_modules:
            loader = pkgutil.find_loader(module)
            if loader is None:
                raise ModuleNotFoundError(
                    f"Error loading plugin named module ({module}). Loader does not exist."
                )
            found_plugins.update(self.find_plugins_by_module_name(module))

        matching_plugins = self.find_matching_plugins()
        if matching_plugins:
            found_plugins.update(matching_plugins)

        if found_plugins:
            self.construct_plugins_dict(found_plugins)

    def find_plugins_by_module_name(self, module: str):
        found_module = importlib.import_module(module)
        plugins = self.find_plugins_from_module(found_module)

        return plugins

    def find_matching_plugins(self):
        matching_modules = [
            importlib.import_module(name)
            for finder, name, ispkg in pkgutil.iter_modules()
            if name.startswith(self.plugin_search_string)
        ]

        plugins = {}

        # todo: I'm sure there's a better way
        for module in matching_modules:
            for submodule_search_loc in module.__spec__.submodule_search_locations:
                plugins.update(self.find_plugins_from_path(submodule_search_loc))

        return plugins

    def find_plugins_from_path(self, module_path: str):
        plugin_dir = Path(module_path).resolve()
        if not Path(module_path).exists():
            raise FileNotFoundError(f"Plugin module: {module_path} does not exist")
        plugins = {}
        for filename in plugin_dir.iterdir():
            filename: Path
            if filename.suffix == ".py":
                module_name = (
                    str(filename.relative_to(plugin_dir.parent))
                    .replace("\\", ".")
                    .rstrip(".py")
                )

                spec = importlib.util.spec_from_file_location(
                    module_name, str(filename)
                )
                module = importlib.util.module_from_spec(spec)
                spec.loader.exec_module(module)

                plugins.update(self.find_plugins_from_module(module))

        return plugins

    def find_plugins_from_module(self, module) -> dict:
        results = {}
        for name, obj in inspect.getmembers(module):
            if (
                inspect.isclass(obj)
                and issubclass(obj, self.base_class)
                and obj != self.base_class
            ):
                results[obj.name] = obj

        return results

    @property
    def plugins(self):
        return {k: v.get("class")() for k, v in self._plugins.items()}

    def construct_plugins_dict(self, found_plugins: dict):
        config_plugins = self.plugin_config.get("plugin_kwargs")
        if found_plugins:
            for name, cls in found_plugins.items():
                registered_name = cls.name
                if issubclass(cls, self.base_class):
                    self._plugins[registered_name] = {"class": cls, "kwargs": {}}
                if config_plugins:
                    matching_config = config_plugins.get(registered_name)
                    if matching_config:
                        self._plugins[registered_name]["kwargs"] = matching_config

    def get_plugin(self, plugin_name: str):
        return self._plugins.get(plugin_name).get("class")

    def invoke_plugin(self, plugin_name: str, *args, **kwargs):
        plugin = self._plugins.get(plugin_name)
        if not plugin:
            raise KeyError(f"Plugin '{plugin_name}' not found in the plugin manager")

        plugin_kwargs = {}
        if plugin and plugin.get("class") and hasattr(plugin.get("class"), "kwargs"):
            plugin_kwargs.update(plugin.get("class").kwargs)
            plugin_kwargs.update(plugin.get("kwargs"))
        plugin_kwargs.update(kwargs)

        return plugin["class"]()(*args, **plugin_kwargs)



import datetime
from pathlib import Path
from zenplate.plugins.plugin_manager import PluginManager

import os

plugin_manager = PluginManager()

plugin_config = {
    "named_modules": [
        "zenplate.plugins.default_plugins.data_plugins",
        "zenplate.plugins.default_plugins.jinja_plugins",
    ],
}


def test_data_default_variables():
    plugin_manager.load_plugins(plugin_config)
    assert plugin_manager.invoke_plugin("defaults")
    assert plugin_manager.invoke_plugin("defaults")["os"]["name"] == os.name


def test_list_path_contents():
    plugin_manager.load_plugins(plugin_config)
    assert plugin_manager.invoke_plugin("ls", Path(__file__).parent) is not None
    assert plugin_manager.invoke_plugin("ls", Path(__file__).parent)


def test_jinja_test_path_exists():
    plugin_manager.load_plugins(plugin_config)
    assert (
        plugin_manager.invoke_plugin("path_exists", Path(__file__).resolve())
        is not None
    )
    assert plugin_manager.invoke_plugin("path_exists", Path(__file__).resolve())


def test_jinja_filter_now():
    plugin_manager.load_plugins(plugin_config)
    output = plugin_manager.invoke_plugin("now", fmt="%Y-%m-%d")
    assert output is not None
    assert output == datetime.datetime.now().strftime("%Y-%m-%d")


def test_jinja_filter_datetime_format():
    plugin_manager.load_plugins(plugin_config)
    output = plugin_manager.invoke_plugin(
        "datetime_format", datetime.datetime.now(), fmt="%Y-%m-%d"
    )
    assert output is not None
    assert output == datetime.datetime.now().strftime("%Y-%m-%d")


def test_jinja_filter_to_yaml():
    plugin_manager.load_plugins(plugin_config)
    output = plugin_manager.invoke_plugin("to_yaml", {"test": "test"})
    assert output is not None
    assert output == "test: test\n"


if __name__ == "__main__":
    test_data_default_variables()
    test_list_path_contents()
    test_jinja_test_path_exists()
    test_jinja_filter_now()
    test_jinja_filter_datetime_format()
    test_jinja_filter_to_yaml()



import os

from zenplate.template_data import TemplateData

from config_fixtures import new_config, fixtures


def test_template_vars_initializes_with_config():
    config = new_config()
    config.jinja_global_vars = {"global_var": "value"}
    template_vars = TemplateData(config)
    assert template_vars.vars["global_var"] == "value"


def test_template_vars_loads_plugins():
    config = new_config()
    template_vars = TemplateData(config)
    template_vars.load_plugins()

    assert "defaults" in template_vars.vars.keys()
    assert template_vars.vars.get("defaults").get("os").get("name") == os.name


def test_template_vars_loads_file():
    config = new_config()
    template_vars = TemplateData(config)
    var_file = [fixtures / "vars" / "vars.yaml"]
    template_vars.load_files(var_file)
    assert template_vars.vars.get("test") == "a"
    assert template_vars.vars.get("test2") == "b"


def test_template_vars_loads_variables():
    config = new_config()
    template_vars = TemplateData(config)
    variables = ["key1=value1", "key2=value2"]
    template_vars.load(variables)


def test_template_vars_handles_invalid_variable_format():
    config = new_config()
    template_vars = TemplateData(config)
    variables = ["invalid_variable"]
    template_vars.load(variables)
    assert "invalid_variable" not in template_vars.vars.keys()


if __name__ == "__main__":
    test_template_vars_initializes_with_config()
    test_template_vars_loads_plugins()
    test_template_vars_loads_file()
    test_template_vars_loads_variables()
    test_template_vars_handles_invalid_variable_format()



import pathlib
import yaml


class PathObject(pathlib.WindowsPath):
    pass


def pathobj_rep(dumper: yaml.Dumper, data):
    if isinstance(data, pathlib.Path):
        data = str(data.resolve())
    return dumper.represent_scalar("tag:yaml.org,2002:str", data)



from jinja2.exceptions import TemplateNotFound


from zenplate.template_manager import TemplateManager
from config_fixtures import new_config, fixtures


def template_manager_loads_template():
    config = new_config()
    config.template_path = fixtures / "templates" / "template.html"
    template_manager = TemplateManager(config)
    assert template_manager.render_template() is not None


def template_manager_raises_error_for_missing_template():
    config = new_config()
    config.template_path = fixtures / "templates" / "non_existent_template.html"
    template_manager = TemplateManager(config)
    try:
        template_manager.render_template()
        assert False, "Expected an exception for missing template"
    except FileNotFoundError:
        pass
    except TemplateNotFound:
        pass


def template_manager_handles_empty_template():
    config = new_config()
    config.template_path = fixtures / "templates" / "empty_template.html"
    template_manager = TemplateManager(config)
    template = template_manager.render_template().get("content")
    assert template is None, f"Expected empty template content, got {template}"


if __name__ == "__main__":
    template_manager_loads_template()
    template_manager_raises_error_for_missing_template()
    template_manager_handles_empty_template()



import logging
from pathlib import Path
from typing import List, Optional

import yaml

from zenplate.plugins.plugin_manager import PluginManager
from zenplate.plugins import DataPlugin


logger = logging.getLogger(__name__)


class TemplateData(object):
    def __init__(self, config):
        self.config = config
        self.vars = {}
        if config.jinja_global_vars:
            self.vars.update(config.jinja_global_vars)
        self.load_plugins()

    def load_plugins(self):
        if self.config.plugin_config:
            try:
                data_plugin_manager = PluginManager(DataPlugin)
                data_plugin_manager.load_plugins(self.config.plugin_config)
                data = {
                    k: data_plugin_manager.invoke_plugin(k)
                    for k, v in data_plugin_manager.plugins.items()
                }

                self.vars.update(data)

            except Exception as e:
                logger.error(f"Error loading data plugins: {e}")
                raise

    def load_files(self, var_files: Optional[List[Path]] = None):
        for var_file in var_files:
            if Path(var_file).exists():
                try:
                    with open(var_file, "r") as file:
                        self.vars.update(yaml.safe_load(file))
                except Exception as e:
                    logger.error(e)

    def load(self, variables: Optional[List[str]] = None):
        if variables:
            for v in variables:
                try:
                    if "=" not in v:
                        logger.warning(
                            f"Variable '{v}' is not parsable as a key:value pair, skipping."
                        )
                        continue
                    split_var = v.split("=")
                    if len(split_var) > 1 and r"\," in split_var[1]:
                        split_var[1] = split_var[1].split(r"\,")
                    self.vars[split_var[0]] = split_var[1]
                except ValueError as e:
                    logger.error(f"Variable {v} could not be parsed\n\t{e}")



from time import sleep

from zenplate.config import Config

from config_fixtures import fixtures


def test_config_init():
    config = Config()

    assert config.config_file is None
    assert config.tree_directory is None
    assert config.var_files == []
    assert config.variables == []
    assert config.log_path is None
    assert config.template_path is None
    assert config.tree_directory is None
    assert config.output_path is None
    assert config.jinja_config == {
        "trim_blocks": False,
        "lstrip_blocks": False,
        "keep_trailing_newline": False,
    }
    assert config.jinja_global_vars == {}
    assert not config.dry_run
    assert config.log_level == "ERROR"
    assert not config.stdout
    assert not config.force_overwrite
    assert config.plugin_config == {
        "path_modules": [],
        "named_modules": [
            "zenplate.plugins.default_plugins.data_plugins",
            "zenplate.plugins.default_plugins.jinja_plugins",
        ],
        "plugin_kwargs": {
            "ls": {
                "path": ".",
            }
        },
    }


def test_config_configure_from_path():
    config = Config()
    config2 = Config()

    config_path = fixtures.parent.joinpath("output", "exported_config.yml")
    config.config_file = config_path
    config.export_config()

    sleep(1)
    config.configure_from_path(config_path)
    non_matching = [
        k
        for k, v in config2.__dict__.items()
        if k in config.__dict__.keys()
        and k != "config_file"
        and config.__dict__[k] != config2.__dict__[k]
    ]

    if non_matching:
        raise AssertionError(f"Non-matching keys: {non_matching}")


def test_config_export_config():
    config = Config()
    default_config_file = fixtures / "configs" / "from_defaults.yml"
    config.configure_from_path(default_config_file)
    config.config_file = fixtures / "output" / "exported_config.yml"

    config.export_config()

    assert default_config_file.read_text() == config.config_file.read_text()


if __name__ == "__main__":
    test_config_init()
    test_config_configure_from_path()
    test_config_export_config()



import json
import pathlib
from pathlib import Path
from typing import Optional, List
from typing_extensions import Annotated, get_type_hints

import yaml

from zenplate.setup_logging import setup_logging
from zenplate.yaml_extras import pathobj_rep


class Config:
    config_file: Annotated[Path, "exclude"] = None

    template_path: Annotated[Path, "exclude"] = None
    tree_directory: Annotated[Path, "exclude"] = None
    output_path: Annotated[Path, "exclude"] = None

    var_files: Optional[List[Path]] = None
    variables: Optional[List[str]] = []
    plugin_config: dict = {}

    log_path: Optional[Path] = None
    log_level: str = "ERROR"

    def __init__(self):
        self.config_file: Annotated[Path, "exclude"] = None
        self.template_path: Annotated[Path, "exclude"] = None
        self.tree_directory: Annotated[Path, "exclude"] = None
        self.output_path: Annotated[Path, "exclude"] = None
        self.var_files = []
        self.jinja_config = {
            "trim_blocks": False,
            "lstrip_blocks": False,
            "keep_trailing_newline": False,
        }
        self.jinja_global_vars: dict = {}
        self.dry_run: bool = False
        self.log_level: str = "ERROR"
        self.log_path: Optional[Path] = None
        self.stdout: bool = False
        self.force_overwrite: bool = False

        default_plugin_config = {
            "path_modules": [],
            "named_modules": [
                "zenplate.plugins.default_plugins.data_plugins",
                "zenplate.plugins.default_plugins.jinja_plugins",
            ],
            "plugin_kwargs": {
                "ls": {
                    "path": ".",
                }
            },
        }
        self.plugin_config = default_plugin_config

        setup_logging(log_path=self.log_path, log_level=self.log_level)

    def attr_exportable(self, key):
        type_hints = get_type_hints(self, include_extras=True).get(key, None)
        if not type_hints:
            return True

        return (
            not hasattr(type_hints, "__metadata__")
            or type_hints.__metadata__[0] != "exclude"
        )

    @property
    def exportable_attrs(self):
        return [
            k
            for k, v in self.__dict__.items()
            if self.attr_exportable(k)
            and not k.startswith("_")
            and not k == "config_file"
        ]

    def configure_from_path(self, config_file_path):
        """
        - Reads in a yaml config file, load top level dictionary keys that match the config class'set of attrs.
        - Will ignore any keys that are contained in the 'self.disallowed_file_attrs' list.
        - Any attrs ending in 'dir' or 'path' with be cast into a Path object, then resolved.
        """
        self.config_file = config_file_path
        if isinstance(self.config_file, Path) and self.config_file.exists():
            loaded_config = {}
            if self.config_file and self.config_file.exists():
                if (
                    ".yaml" in self.config_file.suffixes
                    or ".yml" in self.config_file.suffixes
                ):
                    with open(self.config_file, "r") as file:
                        loaded_config = yaml.safe_load(file)
                elif ".json" in self.config_file.suffixes:
                    with open(self.config_file, "r") as file:
                        loaded_config = json.load(file)
                if loaded_config:
                    for attr, value in loaded_config.items():
                        if hasattr(self, attr) and attr in self.exportable_attrs:
                            if (
                                attr.endswith("dir")
                                or attr.endswith("path")
                                or attr.endswith("file")
                            ):
                                if value is not None:
                                    try:
                                        value = Path(loaded_config[attr]).resolve()
                                    except TypeError as e:
                                        print(f"Error with {attr}: {e}")
                            setattr(self, attr, value)

        setup_logging()

    def export_config(self):
        """
        - Exports the post-configuration config parameters to ./config_export.yml
        """
        export_dict = {
            k: getattr(self, k)
            for k, v in self.__dict__.items()
            if k in self.exportable_attrs
        }

        out_path = Path.cwd().joinpath("zenplate_config_export.yml").resolve()
        if self.config_file:
            out_path = self.config_file.resolve()

        if not out_path.exists():
            if not out_path.parent.exists():
                out_path.parent.mkdir(parents=True, exist_ok=True)
            out_path.touch(0o777, exist_ok=True)

        with open(out_path, "w") as file:
            yaml.add_representer(pathlib.PosixPath, pathobj_rep)
            yaml.add_representer(pathlib.WindowsPath, pathobj_rep)
            yaml.dump(export_dict, file, sort_keys=False)



import os
import sys
from pathlib import Path

from zenplate.plugins.base import plugin_wrapper
from zenplate.plugins.data_plugins import DataPlugin


@plugin_wrapper("ls", DataPlugin)
def list_path_contents(path: str, suffix: str = None):
    path_contents = Path(path).iterdir()
    if suffix:
        path_contents = [i for i in path_contents if i.suffix == suffix]
    return [str(i.resolve()) for i in path_contents]


@plugin_wrapper("defaults", DataPlugin)
def data_default_variables(*args, **kwargs):
    variables = {}
    os_vars = {
        "os": {
            "name": os.name,
        }
    }
    variables.update(os_vars)

    sys_vars = {
        "sys": {
            "defaultencoding": sys.getdefaultencoding(),
            "platform": sys.platform,
            "winver": sys.getwindowsversion().major
            if sys.platform == "win32"
            else None,
        }
    }
    variables.update(sys_vars)

    env = {"environ": {}}
    env["environ"].update(os.environ)
    variables.update(env)

    return variables



from datetime import datetime
from pathlib import Path

from yaml import dump

from zenplate.plugins.base import plugin_wrapper
from zenplate.plugins.jinja_plugins import JinjaFilterPlugin, JinjaTestPlugin


@plugin_wrapper("path_exists", JinjaTestPlugin)
def jinja_test_path_exists(path: str):
    return Path(path).exists()


@plugin_wrapper("now", JinjaFilterPlugin)
def jinja_filter_now(fmt="%Y-%m-%d %H:%M:%S"):
    return datetime.now().strftime(fmt)


@plugin_wrapper("datetime_format", JinjaFilterPlugin)
def jinja_filter_datetime_format(value: datetime, fmt: str = "%d-%m-%y %H:%M"):
    return value.strftime(fmt)


@plugin_wrapper("to_yaml", JinjaFilterPlugin)
def jinja_filter_to_yaml(value):
    return dump(value)



from slugify import slugify
from re import compile

from zenplate.plugins import plugin_wrapper, JinjaTestPlugin, JinjaFilterPlugin

python_path_regex = r"[^-a-z0-9_]+.py"


@plugin_wrapper("valid_python_module_name", JinjaTestPlugin)
def valid_python_module_name(string: str):
    # Admittedly, this regex is not correct
    return compile(python_path_regex).match(string) is not None


@plugin_wrapper("make_slug", JinjaFilterPlugin)
def make_slug(string: str):
    slug = slugify(string, separator="_", regex_pattern=python_path_regex)
    if not slug.endswith(".py"):
        slug += ".py"
    return slug



from zenplate.plugins.base import Plugin


class JinjaFilterPlugin(Plugin):
    @classmethod
    def __call__(cls, *args, **kwargs) -> str:
        return str(cls.func(*args, **kwargs))


class JinjaTestPlugin(Plugin):
    @classmethod
    def __call__(cls, *args, **kwargs) -> bool:
        return bool(cls.func(*args, **kwargs))



from typing import Type, Callable


class Plugin:
    func: Callable
    name: str = ""
    kwargs: dict = {}

    def __init__(self):
        if not hasattr(self, "func"):
            raise TypeError("Used without bound function")

    def __repr__(self):
        return self.name

    def __str__(self):
        return self.__repr__()

    @classmethod
    def __call__(cls, *args, **kwargs):
        return cls.func(*args, **kwargs)


def plugin_wrapper(name: str, cls: Type[object], **kwargs):
    """
    Decorator to turn your function into a Plugin object.
    Copy this function and change the value of Plugin for your own base class.
    """

    def decorator(func):
        if not hasattr(cls, "__call__"):
            raise TypeError("Base class does not have a __call__ method")

        class_properties = {
            "__module__": getattr(func, "__module__"),
            "__doc__": func.__doc__,
            "name": name,
            "func": func,
        }
        class_properties.update(kwargs)

        return type(func.__name__, (cls,), class_properties)

    return decorator



from io import StringIO
import sys

from zenplate.output_handler import OutputHandler
from config_fixtures import new_config, fixtures


def test_output_handler_write_file():
    config = new_config()
    config.output_path = fixtures / "output"
    config.force_overwrite = True
    output_handler = OutputHandler(config)
    template_dict = {
        "test_file": {
            "path": fixtures / "output" / "test_file.txt",
            "content": "test content",
        }
    }
    output_handler.write_file(template_dict)
    assert (fixtures / "output" / "test_file.txt").exists()


def test_output_handler_write_tree():
    config = new_config()
    config.output_path = fixtures / "output"
    config.force_overwrite = True
    output_handler = OutputHandler(config)
    template_dict = {
        "test_file": {
            "path": fixtures / "output" / "tree" / "test_tree1.txt",
            "content": "test content",
        },
        "test_file2": {
            "path": fixtures / "output" / "tree" / "test_tree2.txt",
            "content": "test content",
        },
    }
    output_handler.write_tree(template_dict)

    assert all(template_dict[k]["path"].exists() for k, v in template_dict.items())
    assert all(
        template_dict[k]["path"].read_text() == template_dict[k]["content"]
        for k, v in template_dict.items()
    )


def test_output_handler_write_stdout(monkeypatch):
    config = new_config()
    config.output_path = fixtures / "output" / "test_file.txt"
    config.force_overwrite = True
    output_handler = OutputHandler(config)
    mock_stdout = StringIO()

    monkeypatch.setattr(sys, "stdout", mock_stdout)
    output_handler.write_stdout(config.output_path)

    assert config.output_path.read_text() == "test content"
    assert mock_stdout.getvalue() == "test content\n"


if __name__ == "__main__":
    test_output_handler_write_file()
    test_output_handler_write_tree()
    test_output_handler_write_stdout()



import logging

from pathlib import Path


logger = logging.getLogger(__name__)


class OutputHandler(object):
    def __init__(self, config):
        self.config = config

    def write_tree(self, template_dict: dict):
        if not Path(self.config.output_path).exists():
            Path(self.config.output_path).mkdir(mode=755)
        if template_dict:
            for template, properties in template_dict.items():
                name = template
                path = properties.get("path")
                parent_path = path.parent.resolve()
                content = properties.get("content")
                if not parent_path.exists():
                    parent_path.mkdir(mode=755)

                try:
                    logger.info(f"Writing {name}")

                    path.touch(exist_ok=True)
                    path.write_text(content, encoding="utf-8")
                except Exception as e:
                    logger.error(e, f"Could not write {name}")

    def write_file(self, template_dict: dict):
        if template_dict:
            name, properties = template_dict.popitem()
            path = properties.get("path")
            if path.exists() and not self.config.force_overwrite:
                logger.error(
                    f"File '{path.resolve()}' already exists, use --force to overwrite"
                )
                return
            parent_path = path.parent.resolve()
            content = properties.get("content")

            if not parent_path.exists():
                parent_path.mkdir(mode=0o755, parents=True)
            try:
                logger.info(f"Writing {name}")
                path.touch(exist_ok=True, mode=0o755)
                path.write_text(content, encoding="utf-8")
            except Exception as e:
                logger.error(e, f"Could not write {name}")

    def write_stdout(self, output_path: Path):
        print(output_path.read_text())



import logging
from pathlib import Path

from jinja2 import Environment, FileSystemLoader, select_autoescape

from zenplate.config import Config

from zenplate.plugins.plugin_manager import PluginManager
from zenplate.plugins import JinjaFilterPlugin, JinjaTestPlugin

logger = logging.getLogger(__name__)


class TemplateManager(object):
    def __init__(self, config: Config):
        self.config = config

        self.template_path = config.template_path
        self.tree_dir = config.tree_directory
        if self.template_path:
            self.template_parent = Path(self.template_path).parent
            self.template_name = Path(self.template_path)
            self.env = Environment(
                loader=FileSystemLoader(self.template_parent),
                autoescape=select_autoescape(),
            )
        elif self.tree_dir:
            self.tree_dir = Path(config.tree_directory)
            self.env = Environment(
                loader=FileSystemLoader(self.tree_dir),
                autoescape=select_autoescape(),
            )
        self.configure_jinja(config.jinja_config)
        self.load_plugins()

    def configure_jinja(self, jinja_conf: dict):
        # Load jinja settings
        for attr in jinja_conf.keys():
            if hasattr(self.env, attr):
                try:
                    setattr(self.env, attr, jinja_conf[attr])
                except Exception as e:
                    logger.error(e)

    def load_plugins(self):
        if self.config.plugin_config:
            try:
                jinja_filter_plugin_manager = PluginManager(JinjaFilterPlugin)
                jinja_filter_plugin_manager.load_plugins(self.config.plugin_config)
                filters = {k: v for k, v in jinja_filter_plugin_manager.plugins.items()}
                self.env.filters.update(filters)
            except Exception as e:
                logger.error(f"Error loading jinja filter plugins: {e}")

            try:
                jinja_test_plugin_manager = PluginManager(JinjaTestPlugin)
                jinja_test_plugin_manager.load_plugins(self.config.plugin_config)

                tests = {k: v for k, v in jinja_test_plugin_manager.plugins.items()}
                self.env.tests.update(tests)

            except Exception as e:
                logger.error(f"Error loading jinja test plugins: {e}")

    def render_tree(self):
        output_path = Path(self.config.output_path)
        if not output_path.exists() or self.config.force_overwrite:
            template_list = self.env.loader.list_templates()
            output_path_list = [
                self.transform_path_name(path) for path in template_list
            ]

            template_dict = {
                name: {"path": path, "content": self.env.get_template(name).render()}
                for name, path in zip(template_list, output_path_list)
            }
            return template_dict
        else:
            logger.warning(
                f"{output_path} already exists. Use --force flag to write to an existing directory."
            )

    def render_template(self):
        template_name = Path(self.template_path).name
        template_dict = {
            template_name: {
                "path": self.config.output_path,
                "content": self.env.get_template(template_name).render(),
            }
        }
        return template_dict

    def transform_path_name(self, name):
        name_path = Path(self.config.output_path).joinpath(name).resolve()
        templated_path = self.env.from_string(str(name_path)).render()
        return Path(str(templated_path))



from zenplate.plugins.jinja_plugins import JinjaTestPlugin, JinjaFilterPlugin
from zenplate.plugins.base import plugin_wrapper


@plugin_wrapper("jinja_test", JinjaTestPlugin)
def jinja_test(string: str):
    return isinstance(string, str)


@plugin_wrapper("jinja_filter", JinjaFilterPlugin)
def jinja_filter(a_list: list):
    return [i for i in a_list if len(i) > 1]




import logging
import sys

from zenplate.config import Config
from zenplate.template_manager import TemplateManager
from zenplate.template_data import TemplateData
from zenplate.output_handler import OutputHandler


logger = logging.getLogger(__name__)


def main(config: Config):
    # Create the Jinja environment

    templater = TemplateManager(config)

    # Load the TemplateVars object with the app config
    template_vars = TemplateData(config)

    # Load template_vars object in order of least to most precedence
    if config.var_files:
        template_vars.load_files(config.var_files)
    if config.variables:
        template_vars.load(config.variables)

    # Load the template_vars variable dictionary into the templates environment
    templater.env.globals.update(template_vars.vars)

    if config.dry_run:
        sys.exit(0)

    # Initialize the file handler
    output_handler = OutputHandler(config)

    if templater.template_path:
        template_dict = templater.render_template()
        if not template_dict:
            logger.error("No template data was rendered")
            sys.exit(1)
        properties = list(template_dict.values())[0]
        output_path = properties.get("path")

        output_handler.write_file(template_dict)
        if config.stdout:
            output_handler.write_stdout(output_path)

    elif templater.tree_dir:
        template_dict = templater.render_tree()
        output_handler.write_tree(template_dict)



from .base import plugin_wrapper, Plugin
from .data_plugins import DataPlugin
from .jinja_plugins import JinjaFilterPlugin, JinjaTestPlugin

__all__ = [plugin_wrapper, Plugin, DataPlugin, JinjaFilterPlugin, JinjaTestPlugin]



import logging
import logging.config


def setup_logging(log_level: str = "ERROR", log_path: str = None):
    handlers = ["console"]
    if log_path is not None:
        handlers.append("file")

    log_config = {
        "version": 1,
        "disable_existing_loggers": True,
        "formatters": {
            "default": {
                "format": "%(process)d:%(filename)s:%(module)s.%(funcName)s:%(lineno)d:%(message)s",
            },
            "rich": {
                "format": "(%(name)s)[/] %(message)s",
            },
        },
        "handlers": {
            "console": {"()": "rich.logging.RichHandler", "formatter": "rich"},
        },
        "loggers": {
            "zenplate": {"handlers": handlers, "level": log_level, "propagate": True},
        },
    }
    if "file" in handlers:
        log_config["handlers"]["file"] = {
            "class": "logging.FileHandler",
            "formatter": "default",
            "filename": log_path,
            "mode": "w",
        }

    logging.config.dictConfig(log_config)
